pro Create_Mock_Stars, Event
;
;	uses frame info and PSF info to create images with mock stars
;	and optionally adds the created images to the input images to
;	create modified output images
;
;	DPC		20150813
;			20150818	photometry working, adding polarimetry today
;			20170114	imported from stand-alone into Mock Star Tool
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
s_ptr = PTR_struct.PTR_summary
w_ptr = PTR_struct.PTR_wid
;
;	collect lots of flag information from the gui structure
;
hwp_flag = 1 - (*w_ptr).No_HWP_flag		; 0 means deep phot only; 1 means all 17 images
pol_flag = 1							; 0 means no polarimetry info added; 1 means do full pol
reg_flag = (*w_ptr).Save_DS9_Reg_flag	; 1 means to also save a DS9 reg file for the Deep image
flag_50_50 = (*w_ptr).Half_Half_flag	; 1 means every other mock star has P=0
;
n_mock = (*w_ptr).Nstars				; number of mock stars to add
;										; *** if Nstars -> 0, then don't add mock stars, just copy files
;
mag_0 = (*w_ptr).M0						; bright mock star limit						;
mag_1 = (*w_ptr).M1						; faint mock star limit
;
mag_offset = (*w_ptr).Mag_Offset		; offset magnitude to add to mock stars to put them
										; on the same photometric system as the real stars in the images
hwp_mag_offset = 3.0103					; 16x fainter
;
;mag_pwr_law = 0.375
mag_pwr_law = (*w_ptr).beta				; power law index for magnitude distribution, 0 = uniform
										; any other value is the coeff of: Log(N(m)dm) = a + coeff * m
;
p0 = (*w_ptr).P0						; polarization lower limit
p1 = (*w_ptr).P1						; polarization upper limit
pa0 = (*w_ptr).EPA0						; position angle lower limit
pa1 = (*w_ptr).EPA1						; position angle upper limit
;
;	set up the perfmeter, set all to yellow
;
if( hwp_flag eq 1) then colors = strarr(17) else colors = strarr(1)
colors[*] = 'yellow'
LED_stack, (*w_ptr).Creation_Perfmeter_Window_wid,colors
;
;root_dir = 'C:\Users\Dan\Desktop\papers\2016\1_Clemens+_GPIPS_Large-Scale\figures\Figure_0_Mock_Stars\'
;
;	which GP field - kludge for now /*****
;
;gp_dir = dialog_pickfile(Title='Select GP Group Directory',/dir)
;
root_dir = (*w_ptr).Current_Path + (*w_ptr).Current_Trial_Name + PATH_SEP()
;
;  *******/
;
input_dir = 	root_dir + 'Input_Images'
;
print_text, Event, 'Directory of Input Substrate Images = ' + input_dir
;
psf_model_dir = root_dir + 'PSF_Models'
;
print_text, Event, 'Directory of Input PSF Models = ' + psf_model_dir
;
;run_dir = dialog_pickfile(Title='Select Run Subdirectory',/dir,path=root_dir)
run_dir = root_dir + 'Run_' + STRING( (*w_ptr).Run_Number, format='(I2.2)')
;
;	if this run directory doesn't exist, create it
;
result = file_test(run_dir,/DIRECTORY)
if(result eq 0) then begin
	FILE_MKDIR,run_dir
	print_text, Event, 'Created new Run Directory = ' + run_dir
endif
;
;	if Nstars > 0 create mock dir
;
if(n_mock gt 0) then begin
	mock_dir = 		run_dir + PATH_SEP() + 'Mock_Images'
	;
	; if mock_dir not present, make it
	;
	result = file_test(mock_dir,/DIRECTORY)
	if(result eq 0) then begin
		FILE_MKDIR,mock_dir
		print_text, Event, 'Created new Output Directory for Mock Star Images = ' + mock_dir
	endif else begin
		print_text, Event, 'Output Directory for Mock Star Images = ' + mock_dir
	endelse
endif else begin
	print_text, Event, 'No Mock Stars to be generated, so no Mock Image directory created'
endelse
;
output_dir = 	run_dir + PATH_SEP() + 'Output_Images'
;
;	if output directory not present, make it
;
result = FILE_TEST(output_dir,/DIRECTORY)
if(result eq 0) then begin
	FILE_MKDIR, output_dir
	print_text, Event, 'Created new Output Directory for Mock+Real Star Images = ' + output_dir
endif else begin
	print_text, Event, 'Output Directory for Mock+Real Images = ' + output_dir
endelse
;
;---------------------------------------------------------------------------------------------------------
;
;	go to input directory and capture the GP number of the field(s) represented
;
files = FILE_SEARCH(input_dir,'*.fits')
;
;	how many of these have "Deep_Coadd" in their names?
;
ind_deep = WHERE(STRPOS(files,'Deep_Coadd') ne -1, ndeep)
;
print_text, Event, 'Found '+STRING(ndeep,format='(I)')+' Deep_Coadd FITS file'
;
;	loop on each GP from here - this SHOULD be only one, in this gui implementation
;
for igp = 0, ndeep-1 do begin
	;
	;	which GP?
	;
	this_deep_file = files[ind_deep[igp]]
	reads, FILE_BASENAME(this_deep_file),gpnum,format='(2x,I4)'
	;
	;	create observation group name
	;
	obs_grp_name = 'GP'+STRING(gpnum,format='(I4.4)')
	;
	print_text, Event, ".. Starting Mock Star Generation for field "+obs_grp_name
	;
;	stop
	;
	;	read the image to get x, y sizes
	;
	deep_image = readfits(this_deep_file, deep_header,/SILENT)
	;
	mock_image = deep_image * 0.0
	msize = SIZE(mock_image)
	xsize = msize[1]
	ysize = msize[2]
	;
	;------------------------------------------------------------------------------------
	;
	;	create the random x, y location of the mock stars
	;		use the x, y sizes, but stay away from the outer 50 pixels in all directions
	;
	if(n_mock gt 0) then begin
		xlow = 50.0
		xhigh = xsize - 51.0
		ylow = 50.0
		yhigh = ysize - 51.0
		;
		iseed = systime(1)
		;
		;	magnitudes -
		;		if index = 0, uniform
		;
		if(mag_pwr_law eq 0.0) then begin
			magmock = mag_0 + (mag_1 - mag_0) * RANDOMU(iseed, n_mock)
			;
		endif else begin
			;
			;	properly model the random magmock values
			;
			make_magmock, event, magmock, mag_0, mag_1, n_mock, mag_pwr_law, iseed
			;
			;	updated number of mock stars
			;
			new_n_mock = N_ELEMENTS(magmock)
			;
		endelse
		;
		n_mock = new_n_mock
		;
		xmock = xlow + (xhigh - xlow) * RANDOMU(iseed, n_mock)
		ymock = ylow + (yhigh - ylow) * RANDOMU(iseed, n_mock)
		;
		;	ok, have x,y,mag - next add P, PA if pol flag set
		;
		if(pol_flag eq 1) then begin
			pmock = p0 + (p1 - p0) * RANDOMU(iseed, n_mock)
			pamock = pa0 + (pa1 - pa0) * RANDOMU(iseed, n_mock)
		endif else begin
			pmock = fltarr(n_mock)
			pamock = fltarr(n_mock)
		endelse
		;
		;	if flag_50_50 is set, zero polarizations of every other mock star
		;
		if(flag_50_50 eq 1) then begin
			pmock[0:n_mock-1:2] = 0.0
		endif
		;
		;
		;****** for debugging purposes
	;	n_mock = 2
	;	xmock = xmock[0:1]
	;	ymock = ymock[0:1]
	;	pmock = pmock[0:1]
	;	pamock = pamock[0:1]
		;*****
	endif
	;
	;	for each of the Deep Photometric and the 16 HWP images, create the mock stars and mock image
	;
	if(hwp_flag eq 1) then max_fields = 16 else max_fields = 0
	;
	for ii = 0, max_fields	do begin
		;
		;	Deep first, then 16 HWP
		;
		;	clear the mock image before each creation episode
		;
		mock_image *= 0.0
		;
		case ii of
			0: begin
				;
				colors[0] = 'blue'
				LED_stack, (*w_ptr).Creation_Perfmeter_Window_wid,colors
				;
				if(n_mock gt 0) then begin
					;
					;	restore the PSF model file for the deep image
					;
					psf_model_file = psf_model_dir + PATH_SEP() + 'S8_Deep_PSF_Model.sav'
					restore, filename=psf_model_file
					;
					;	returns S8_PSF_Model structure
					;
					;	account for mag offset - this puts the PSF model onto the same photometric system as the image
					;
					deep_magmock = magmock - mag_offset
					;
					;	create the mock image
					;
					this_hwp = -1
					make_mock_image, mock_image, S8_Deep_PSF_Model, xmock, ymock, deep_magmock, pmock, pamock, this_hwp
					;
					;	save it to disk
					;
					deep_str = 'mock_deep.fits'
					;
					full_path = mock_dir + PATH_SEP() + deep_str
					;
					writefits, full_path, mock_image
					print_text, Event, '..saving mock_deep image to ' + full_path
					;
					;	combine with input image
					;
					combined_image = deep_image + mock_image
					;
					;	write to output directory
					;
					outfile = output_dir + PATH_SEP() + FILE_BASENAME(this_deep_file)
					;
					;	add the mock star modeling info to the FITS header
					;
					SXADDPAR, deep_header, 'HISTORY','Mock Star Tool modified image on '+SYSTIME()
					SXADDPAR, deep_header, 'MST_NSTR',n_mock,'Number of mock stars added'
					SXADDPAR, deep_header, 'MST_BETA',mag_pwr_law,'Beta slope of log(n) histogram'
					SXADDPAR, deep_header, 'MST_M0',mag_0,'Bright limit of mock stars, mag'
					SXADDPAR, deep_header, 'MST_M1',mag_1,'Faint limit of mock stars, mag'
					SXADDPAR, deep_header, 'MST_MOFF',mag_offset,'Offset magnitude added'
					SXADDPAR, deep_header, 'MST_P0',p0,'Low polarization percentage limit, percentage'
					SXADDPAR, deep_header, 'MST_P1',p1,'High polarization percentage limit, percentage'
					SXADDPAR, deep_header, 'MST_EPA0',pa0,'Low polarization position angle limit, deg'
					SXADDPAR, deep_header, 'MST_EPA1',pa1,'High polarization position angle limit, deg'
					SXADDPAR, deep_header, 'MST_NHWP',1-hwp_flag,'No HWP flag; 0=17 images; 1=deep only'
					SXADDPAR, deep_header, 'MST_REGF',reg_flag,'Save DS9 Reg Flag; 1 = yes'
					SXADDPAR, deep_header, 'MST_5050',flag_50_50,'Flag, if set, half P values set to zero'
					;
					writefits, outfile, combined_image, deep_header
					;
					print_text, Event, '...wrote mock+real deep image to '+FILE_BASENAME(outfile)+' in output directory'
				endif else begin
					outfile = output_dir + PATH_SEP() + FILE_BASENAME(this_deep_file)
					writefits, outfile, deep_image, deep_header
					;
					print_text, Event, '...wrote original deep image to '+FILE_BASENAME(outfile)+' in output directory'
				endelse
				;
;				;	save on file list for copying
;				;
;				(*w_ptr).File_Copy_List[0] = outfile
				;
				;	save a file of mock star catalog of properties
				;
				if(n_mock gt 0) then begin
					outmockstars = run_dir + PATH_SEP() + FILE_BASENAME(this_deep_file,'Deep_Coadd.fits') + 'mock_stars.dat'
					openw, lun, outmockstars, /GET_LUN
					;
					;	if reg file flag set, write a DS9 reg file of circled stars (in yellow)
					;
					if(reg_flag eq 1) then begin
						;
					;	reg_file = dialog_pickfile(Title='DS9 Reg file to create',path=run_dir)
						reg_file = run_dir + PATH_SEP() + FILE_BASENAME(this_deep_file,'Deep_Coadd.fits') + '_DS9_reg.reg'
						;
						openw, lu_reg, reg_file, /GET_LUN
						;
					endif
					;
					;	remove mag offset
					;
					deep_magmock += mag_offset
					;
					;	and correct for pixel size
					;
					deep_xmock = xmock - 0.5
					deep_ymock = ymock - 0.5
					;
					;	sort by ra (inverse x)
					;
					ind = REVERSE(SORT(deep_xmock))
					;
					;	save the mock star catalog and information about all the flags
					;
					printf, lun, '; mock star values for ',FILE_BASENAME(this_deep_file),' on ',systime()
					printf, lun, ';   created with model parameters as follow:'
					printf, lun, ';   BETA = ',mag_pwr_law,' - slope of log(n) histo'
					printf, lun, ';   M0, M1 = ',mag_0,mag_1,' - bounding magnitudes'
					printf, lun, ';   MAG_OFFSET = ',mag_offset,' - offset added'
					printf, lun, ';   P0, P1 = ',p0,p1,' - bounding polarization percentages'
					printf, lun, ';   EPA0, EPA1 = ',pa0,pa1,' - bounding position angles'
					printf, lun, ';   NO_HWP_FLAG = ',1-hwp_flag,' - set means deep image, only'
					printf, lun, ';   REG_FLAG = ',reg_flag,' - set means save DS9 reg file of mock star X,Y'
					printf, lun, ';   50/50_FLAG = ',flag_50_50,' - if set, 50% of P values set to 0'
					printf, lun, ';
					printf, lun, ';'
					printf, lun, '; No.     X        Y       mag      P        EPA'
					printf, lun, ';--------------------------------------------------'
					;
					for ik = 0, n_mock-1 do begin
						im = ind[ik]
						printf, lun, ik, deep_xmock[im],deep_ymock[im],deep_magmock[im],pmock[im],pamock[im],$
								format='(I4,3x,F6.1,3x,F6.1,3x,F6.3,3x,F6.3,3x,F6.1)'
						;
						if(reg_flag eq 1) then begin	; add 1 in x and y for DS9
							printf, lu_reg, deep_xmock[im]+1.0,deep_ymock[im]+1.0,format='("circle    ",F6.1,"    ",F6.1,"  4 # color= yellow width=  1")'
						endif
					endfor
					;
					FREE_LUN, lun
					print_text, event, 'Wrote Mock Star Catalog File to disk'
					if(reg_flag eq 1) then begin
						FREE_LUN, lu_reg
						print_text, event, 'Wrote DS9 Reg File to disk'
					endif
				endif
				;
				; change first perfmeter bar to green
				;
				colors[0] = 'green'
				LED_stack, (*w_ptr).Creation_Perfmeter_Window_wid,colors
				;
			   end
			   ;
			   ;	if HWP files, too, do them here
			   ;
		else: begin
				;
				this_hwp = ii - 1
				;
				colors[ii] = 'blue'
				LED_stack, (*w_ptr).Creation_Perfmeter_Window_wid,colors
				;
				;	read the HWP image
				;
				hwp_path = FILE_DIRNAME(this_deep_file) + PATH_SEP() + STRING(gpnum, this_hwp, $
							format='("GP",i4.4,"_HWP_",i2.2,".fits")')
				print_text, event, 'Reading input HWP image '+FILE_BASENAME(hwp_path)
				;
				hwp_image = readfits(hwp_path, hwp_header,/SILENT)
				;
				;	restore the PSF model file for this HWP image
				;
				if(n_mock gt 0) then begin
					psf_model_file = psf_model_dir + PATH_SEP() + 'S8_HWP'+$
										STRING(this_hwp,format='(I2.2)') + '_PSF_Model.sav'
					restore, filename=psf_model_file
					;
					;	returns S8_PSF_Model structure
					;
					;	account for mag offset - this puts the PSF model onto the same photometric system as the image
					;
					HWP_magmock = magmock - mag_offset + hwp_mag_offset	; add 3.0103 mag = 16x
					;
					;	create the mock image
					;
					make_mock_image, mock_image, S8_HWP_PSF_Model, xmock, ymock, HWP_magmock, pmock, pamock, this_hwp
					;
					;	save to disk
					;
					hwp_str = 'mock_hwp' + STRING(this_hwp,format='(I2.2)') + '.fits'
					writefits, mock_dir + PATH_SEP() + hwp_str, mock_image
					print_text, event, 'Wrote HWP Mock Star Image = '+ hwp_str
					;
					;	combine with input image
					;
					combined_image = HWP_image + mock_image
					;
					;	write to output directory
					;
					outfile = output_dir + PATH_SEP() + FILE_BASENAME(hwp_path)
					writefits, outfile, combined_image, hwp_header
					print_text, event, 'Wrote HWP Mock+Real Star Image = ' + FILE_BASENAME(outfile)
				endif else begin
					outfile = output_dir + PATH_SEP() + FILE_BASENAME(hwp_path)
					writefits, outfile, hwp_image, hwp_header
					print_text, event, 'Wrote Original HWP Image = ' + FILE_BASENAME(outfile)
				endelse
;				;
;				;	save on file list for copying
;				;
;				(*w_ptr).File_Copy_List[ii] = outfile
				;
				colors[ii] = 'green'
				LED_stack, (*w_ptr).Creation_Perfmeter_Window_wid,colors
			  end
				;
		endcase
	endfor
	;
	;	this GP group all done, do then next one or quit
	;
endfor
;
end

